<?php

namespace Matrimony\Matrimony\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Matrimony\Matrimony\Http\Requests\PreferenceRequest;
use Matrimony\Matrimony\Interfaces\PreferenceRepositoryInterface;
use Matrimony\Matrimony\Models\Preference;

/**
 * Resource controller class for preference.
 */
class PreferenceResourceController extends BaseController
{

    /**
     * Initialize preference resource controller.
     *
     * @param type PreferenceRepositoryInterface $preference
     *
     * @return null
     */
    public function __construct(PreferenceRepositoryInterface $preference)
    {
        parent::__construct();
        $this->repository = $preference;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Matrimony\Matrimony\Repositories\Criteria\PreferenceResourceCriteria::class);
    }

    /**
     * Display a list of preference.
     *
     * @return Response
     */
    public function index(PreferenceRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Matrimony\Matrimony\Repositories\Presenter\PreferencePresenter::class)
                ->$function();
        }

        $preferences = $this->repository->paginate();

        return $this->response->title(trans('matrimony::preference.names'))
            ->view('matrimony::preference.index', true)
            ->data(compact('preferences'))
            ->output();
    }

    /**
     * Display preference.
     *
     * @param Request $request
     * @param Model   $preference
     *
     * @return Response
     */
    public function show(PreferenceRequest $request, Preference $preference)
    {

        if ($preference->exists) {
            $view = 'matrimony::preference.show';
        } else {
            $view = 'matrimony::preference.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('matrimony::preference.name'))
            ->data(compact('preference'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new preference.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PreferenceRequest $request)
    {

        $preference = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('matrimony::preference.name')) 
            ->view('matrimony::preference.create', true) 
            ->data(compact('preference'))
            ->output();
    }

    /**
     * Create new preference.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PreferenceRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $preference                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('matrimony::preference.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('matrimony/preference/' . $preference->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/matrimony/preference'))
                ->redirect();
        }

    }

    /**
     * Show preference for editing.
     *
     * @param Request $request
     * @param Model   $preference
     *
     * @return Response
     */
    public function edit(PreferenceRequest $request, Preference $preference)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('matrimony::preference.name'))
            ->view('matrimony::preference.edit', true)
            ->data(compact('preference'))
            ->output();
    }

    /**
     * Update the preference.
     *
     * @param Request $request
     * @param Model   $preference
     *
     * @return Response
     */
    public function update(PreferenceRequest $request, Preference $preference)
    {
        try {
            $attributes = $request->all();

            $preference->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('matrimony::preference.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('matrimony/preference/' . $preference->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('matrimony/preference/' . $preference->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the preference.
     *
     * @param Model   $preference
     *
     * @return Response
     */
    public function destroy(PreferenceRequest $request, Preference $preference)
    {
        try {

            $preference->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('matrimony::preference.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('matrimony/preference/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('matrimony/preference/' . $preference->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple preference.
     *
     * @param Model   $preference
     *
     * @return Response
     */
    public function delete(PreferenceRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('matrimony::preference.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('matrimony/preference'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/matrimony/preference'))
                ->redirect();
        }

    }

    /**
     * Restore deleted preferences.
     *
     * @param Model   $preference
     *
     * @return Response
     */
    public function restore(PreferenceRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('matrimony::preference.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/matrimony/preference'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/matrimony/preference/'))
                ->redirect();
        }

    }

}
